@@ -7,7 +7,9 @@ no warnings;
 use subs qw();
 use vars qw( $VERSION );
 
-$VERSION = '1.96';
+$VERSION = '2.001';
+
+=encoding utf8
 
 =head1 NAME
 
@@ -15,8 +17,12 @@ Set::CrossProduct - work with the cross product of two or more sets
 
 =head1 SYNOPSIS
 
+	# unlabeled sets
 	my $iterator = Set::CrossProduct->new( ARRAY_OF_ARRAYS );
 
+	# or labeled sets where hash keys are the set names
+	my $iterator = Set::CrossProduct->new( HASH_OF_ARRAYS );
+
 	# get the number of tuples
 	my $number_of_tuples = $iterator->cardinality;
 
@@ -43,6 +49,7 @@ Set::CrossProduct - work with the cross product of two or more sets
 	# in scalar context returns an array reference to all tuples
 	my $tuples           = $iterator->combinations;
 
+
 =head1 DESCRIPTION
 
 Given sets S(1), S(2), ..., S(k), each of cardinality n(1), n(2), ..., n(k)
@@ -77,7 +84,7 @@ tuples shown:
 	( b, 2, foo )
 	( b, 2, bar )
 	( b, 3, foo )
-	( b, 3, bar ) 
+	( b, 3, bar )
 	( c, 1, foo )
 	( c, 1, bar )
 	( c, 2, foo )
@@ -90,51 +97,75 @@ too.
 
 	A => ( a, b, c )
 	B => ( )
-	
+
 In this case, A x B is the empty set, so you'll get no tuples.
 
 This module combines the arrays that give to it to create this
 cross product, then allows you to access the elements of the
 cross product in sequence, or to get all of the elements at
-once.  Be warned! The cardnality of the cross product, that is,
+once. Be warned! The cardinality of the cross product, that is,
 the number of elements in the cross product, is the product of
 the cardinality of all of the sets.
 
 The constructor, C<new>, gives you an iterator that you can
-use to move around the cross product.  You can get the next
+use to move around the cross product. You can get the next
 tuple, peek at the previous or next tuples, or get a random
-tuple.  If you were inclined, you could even get all of the
+tuple. If you were inclined, you could even get all of the
 tuples at once, but that might be a very large list. This module
 lets you handle the tuples one at a time.
 
 I have found this module very useful for creating regression
-tests.  I identify all of the boundary conditions for all of
+tests. I identify all of the boundary conditions for all of
 the code branches, then choose bracketing values for each of them.
 With this module I take all of the values for each test and
 create every possibility in the hopes of exercising all of the
-code.  Of course, your use is probably more interesting. :)
+code. Of course, your use is probably more interesting. :)
 
 =head1 METHODS
 
-=head2 new( ARRAY_REF_OF_ARRAY_REFS )
+=over 4
 
-Given the array of arrays that represent some sets, return a
-C<Set::CrossProduct> instance that represents the cross product
-of those sets.
+=item * new( [ [ ... ], [ ... ] ])
 
-The single argument is an array reference that has as its
-elements other array references.  The C<new> method will
-return undef in scalar context and the empty list in list
-context if you give it something different.
+=item * new( { LABEL => [ ... ], LABEL2 => [ ... ] } )
 
-You must have at least two sets, or the constructor will
-fail.
+Given arrays that represent some sets, return a C<Set::CrossProduct>
+instance that represents the cross product of those sets.
+
+You can create the sets in two different ways: unlabeled and labeled sets.
+
+For unlabeled sets, you don't give them names. You rely on position. To
+create this, pass an array of arrays:
+
+	my $unlabeled = Set::CrossProduct->new( [
+		[ qw(1 2 3) ],
+		[ qw(a b c) ],
+		[ qw(! @ $) ],
+		] );
+
+When you call C<next>, you get an array ref where the positions in the
+tuple correspond to the position of the sets you gave C<new>:
+
+	my $tuple = $unlabeled->next;   #  [ qw(1 a !) ]
+
+For labeled sets, you want to give each set a name. When you ask for a tuple,
+you get a hash reference with the labels you choose:
+
+	my $labeled = Set::CrossProduct->new( {
+		number => [ qw(1 2 3) ],
+		letter => [ qw(a b c) ],
+		symbol => [ qw(! @ $) ],
+		} );
+
+	my $tuple = $labeled->next;   #  { number => 1, letter => 'a', symbol => '!' }
 
 =cut
 
 # The iterator object is a hash with these keys
 #
 #	arrays   - holds an array ref of array refs for each list
+#   labels   - the names of the set, if applicable
+#   labeled  - boolean to note if the sets are labeled or not
 #	counters - the current position in each array for generating
 #		combinations
 #	lengths  - the precomputed lengths of the lists in arrays
@@ -144,20 +175,33 @@ fail.
 #	ungot    - true if we just ungot something--to prevent
 #		attempts at multiple ungets which we don't support
 
-sub new
-	{
-	my( $class, $array_ref ) = @_;
+sub new {
+	my( $class, $constructor_ref ) = @_;
 
-	return unless ref $array_ref eq ref [];
+	my $ref_type = ref $constructor_ref;
+
+	my $self = {};
+
+	if( $ref_type eq ref {} ) {
+		$self->{labeled} = 1;
+		$self->{labels}  = [ sort keys %$constructor_ref ];
+		$self->{arrays}  = [ @$constructor_ref{ sort keys %$constructor_ref } ];
+		}
+	elsif( $ref_type eq ref [] ) {
+		$self->{labeled} = 1;
+		$self->{arrays}  = $constructor_ref;
+		}
+	else {
+		return;
+		}
+
+	my $array_ref = $self->{arrays};
 	return unless @$array_ref > 1;
 
 	foreach my $array ( @$array_ref ) {
 		return unless ref $array eq ref [];
 		}
 
-	my $self = {};
-
-	$self->{arrays}   = $array_ref;
 	$self->{counters} = [ map { 0 }      @$array_ref ];
 	$self->{lengths}  = [ map { $#{$_} } @$array_ref ];
 	$self->{previous} = [];
@@ -171,24 +215,20 @@ sub new
 	return $self;
 	}
 
-sub _increment
-	{
+sub _increment {
 	my $self = shift;
 
 	$self->{previous} = [ @{$self->{counters}} ]; # need a deep copy
 
 	my $tail = $#{ $self->{counters} };
 
-	COUNTERS:
-		{
-		if( $self->{counters}[$tail] == $self->{lengths}[$tail] )
-			{
+	COUNTERS: {
+		if( $self->{counters}[$tail] == $self->{lengths}[$tail] ) {
 			$self->{counters}[$tail] = 0;
 			$tail--;
 
 			if( $tail == 0
-				and $self->{counters}[$tail] == $self->{lengths}[$tail] )
-				{
+				and $self->{counters}[$tail] == $self->{lengths}[$tail] ) {
 				$self->done(1);
 				return;
 				}
@@ -202,8 +242,7 @@ sub _increment
 	return 1;
 	}
 
-sub _decrement
-	{
+sub _decrement {
 	my $self = shift;
 
 	my $tail = $#{ $self->{counters} };
@@ -214,8 +253,7 @@ sub _decrement
 	return 1;
 	}
 
-sub _previous
-	{
+sub _previous {
 	my $self = shift;
 
 	my $counters = $self->{counters};
@@ -224,15 +262,12 @@ sub _previous
 
 	return [] unless grep { $_ } @$counters;
 
-	COUNTERS:
-		{
-		if( $counters->[$tail] == 0 )
-			{
+	COUNTERS: {
+		if( $counters->[$tail] == 0 ) {
 			$counters->[$tail] = $self->{lengths}[$tail];
 			$tail--;
 
-			if( $tail == 0 and $counters->[$tail] == 0)
-				{
+			if( $tail == 0 and $counters->[$tail] == 0) {
 				$counters = [ map { 0 } 0 .. $tail ];
 				last COUNTERS;
 				}
@@ -246,40 +281,47 @@ sub _previous
 	return $counters;
 	}
 
-=head2 cardinality()
+=item * labeled()
+
+Return true if the sets are labeled (i.e. you made the object from
+a hash ref). Returns false otherwise. You might use this to figure out
+what sort of value C<get> will return.
+
+=cut
+
+sub labeled { !! $_[0]->{labeled} }
+
+=item * cardinality()
 
 Return the carnality of the cross product.  This is the number
 of tuples, which is the product of the number of elements in
 each set.
 
 Strict set theorists will realize that this isn't necessarily
-the real cardinality since some tuples may be indentical, making
+the real cardinality since some tuples may be identical, making
 the actual cardinality smaller.
 
 =cut
 
-sub cardinality
-	{
+sub cardinality {
 	my $self = shift;
 
 	my $product = 1;
 
-	foreach my $length ( @{ $self->{lengths} } )
-		{
+	foreach my $length ( @{ $self->{lengths} } ) {
 		$product *= ( $length + 1 );
 		}
 
 	return $product;
 	}
 
-=head2 reset_cursor()
+=item * reset_cursor()
 
 Return the pointer to the first element of the cross product.
 
 =cut
 
-sub reset_cursor
-	{
+sub reset_cursor {
 	my $self = shift;
 
 	$self->{counters} = [ map { 0 } @{ $self->{counters} } ];
@@ -290,50 +332,69 @@ sub reset_cursor
 	return 1;
 	}
 
-=head2 get()
+=item * get()
 
 Return the next tuple from the cross product, and move the position
-to the tuple after it.
-
-In list context, C<get> returns the tuple as a list.  In scalar context
-C<get> returns the tuple as an array reference.
-
-If you have already gotten the last tuple in
+to the tuple after it. If you have already gotten the last tuple in
 the cross product, then C<get> returns undef in scalar context and
 the empty list in list context.
 
+What you get back depends on how you made the constructor.
+
+For unlabeled sets, you get back an array reference in scalar context
+or a list in list context:
+
+For labeled sets, you get back a hash reference in scalar context or a
+list of key-value pairs in list context.
+
 =cut
 
-sub get
-	{
+sub get {
 	my $self = shift;
 
 	return if $self->done;
 
-	my @array = map {  ${ $self->{arrays}[$_] }[ $self->{counters}[$_] ]  }
-			0 .. $#{ $self->{arrays} };
+	my $next_ref = $self->_find_ref('next');
 
 	$self->_increment;
 	$self->{ungot} = 0;
 
-	if( wantarray ) { return  @array }
-	else            { return \@array }
+	if( wantarray ) { return (ref $next_ref eq ref []) ? @$next_ref : %$next_ref }
+	else            { return $next_ref }
 	}
 
-=head2 unget()
+sub _find_ref {
+	my ($self, $which) = @_;
 
-Pretend we did not get the tuple we just got.  The next
-time we get a tuple, we will get the same thing.  You
-can use this to peek at the next value and put it back
-if you do not like it.
+	my $place_func =
+		  ($which eq 'next') ? sub { $self->{counters}[shift] }
+		: ($which eq 'prev') ? sub { $self->{previous}[shift] }
+		: ($which eq 'rand') ? sub { rand(1 + $self->{lengths}[shift]) }
+		:                      undef;
 
-You can only do this for the previous tuple.  C<unget>
-does not do multiple levels of unget.
+	return unless $place_func;
+
+	my @indices = (0 .. $#{ $self->{arrays} });
+
+	if ($self->{labels}) {
+		 return +{ map {  $self->{labels}[$_] => ${ $self->{arrays}[$_] }[ $place_func->($_) ]  } @indices } }
+	else {
+		return [ map {  ${ $self->{arrays}[$_] }[ $place_func->($_) ]  } @indices ]
+		}
+	}
+
+=item * unget()
+
+Pretend we did not get the tuple we just got.  The next time we get a
+tuple, we will get the same thing.  You can use this to peek at the
+next value and put it back if you do not like it.
+
+You can only do this for the previous tuple.  C<unget> does not do
+multiple levels of unget.
 
 =cut
 
-sub unget
-	{
+sub unget {
 	my $self = shift;
 
 	return if $self->{ungot};
@@ -349,58 +410,44 @@ sub unget
 	return 1;
 	}
 
-=head2 next()
+=item * next()
 
-Return the next tuple, but do not move the pointer.  This
-way you can look at the next value without affecting your
-position in the cross product.
-
-In list context, C<get> returns the tuple as a list.  In scalar context
-C<get> returns the tuple as an array reference.
-
-For the last combination, next() returns undef.
+Like C<get>, but does not move the pointer.  This way you can look at
+the next tuple without affecting your position in the cross product.
 
 =cut
 
-sub next
-	{
+sub next {
 	my $self = shift;
 
 	return if $self->done;
 
-	my @array = map( {  ${ $self->{arrays}[$_] }[ $self->{counters}[$_] ]  }
-			0 .. $#{ $self->{arrays} } );
+	my $next_ref = $self->_find_ref('next');
 
-	if( wantarray ) { return  @array }
-	else            { return \@array }
+	if( wantarray ) { return (ref $next_ref eq ref []) ? @$next_ref : %$next_ref }
+	else            { return $next_ref }
 	}
 
-=head2 previous()
+=item * previous()
 
-Return the previous tuple, but do not move the pointer.  This
-way you can look at the last value without affecting your
-position in the cross product.
-
-In list context, C<get> returns the tuple as a list.  In scalar context
-C<get> returns the tuple as an array reference.
+Like C<get>, but does not move the pointer.  This way you can look at
+the previous tuple without affecting your position in the cross product.
 
 =cut
 
-sub previous
-	{
+sub previous {
 	my $self = shift;
 
-	my @array = map( {  ${ $self->{arrays}[$_] }[ $self->{previous}[$_] ]  }
-			0 .. $#{ $self->{arrays} } );
+	my $prev_ref = $self->_find_ref('prev');
 
-	if( wantarray ) { return  @array }
-	else            { return \@array }
+	if( wantarray ) { return (ref $prev_ref eq ref []) ? @$prev_ref : %$prev_ref }
+	else            { return $prev_ref }
 	}
 
-=head2 done()
+=item * done()
 
 Without an argument, C<done> returns true if there are no more
-combinations to fetch with C<get>. and returns false otherwise.
+combinations to fetch with C<get> and returns false otherwise.
 
 With an argument, it acts as if there are no more arguments to fetch, no
 matter the value. If you want to start over, use C<reset_cursor> instead.
@@ -409,47 +456,42 @@ matter the value. If you want to start over, use C<reset_cursor> instead.
 
 sub done { $_[0]->{done} = 1 if @_ > 1; $_[0]->{done} }
 
-=head2 random()
+=item * random()
 
-Return a random tuple from the cross product.
-
-In list context, C<get> returns the tuple as a list.  In scalar context
-C<get> returns the tuple as an array reference.
+Return a random tuple from the cross product. The return value is the
+same as C<get>.
 
 =cut
 
-sub random
-	{
+sub random {
 	my $self = shift;
 
-	my @array = map {  ${ $self->{arrays}[$_] }[ rand(1+$self->{lengths}[$_]) ] }
-			0 .. $#{ $self->{arrays} };
+	my $rand_ref = $self->_find_ref('rand');
 
-	if( wantarray ) { return  @array }
-	else            { return \@array }
+	if( wantarray ) { return (ref $rand_ref eq ref []) ? @$rand_ref : %$rand_ref }
+	else            { return $rand_ref }
 	}
 
-=head2 combinations()
+=item * combinations()
 
-Returns a reference to an arrray that contains all of the tuples
+Returns a reference to an array that contains all of the tuples
 of the cross product.  This can be quite large, so you might
-want to check the cardinality first.
+want to check the cardinality first. The array elements are the
+return values for C<get>.
 
-In list context, C<get> returns the tuple as a list.  In scalar context
-C<get> returns the tuple as an array reference.  However, you should
-probably always use this in scalar context except for very low
-cardnalities to avoid returning huge lists.
+You should probably always use this in scalar context except for
+very low cardinalities to avoid huge return values.
+
+=back
 
 =cut
 
-sub combinations
-	{
+sub combinations {
 	my $self = shift;
 
 	my @array = ();
 
-	while( my $ref = $self->get )
-		{
+	while( my $ref = $self->get ) {
 		push @array, $ref;
 		}
 
@@ -459,22 +501,29 @@ sub combinations
 
 =head1 TO DO
 
-* it would be nice to be able to name the sets, and then access
-elements from a tuple by name, like
-
-	my $i = Set::CrossProduct->new( {
-			Apples => [ ... ],
-			Oranges => [ ... ],
-			}
-		);
-
-	my $tuple = $i->get;
-
-	my $apple = $tuple->Apples;
-
 * I need to fix the cardinality method. it returns the total number
 of possibly non-unique tuples.
 
+* I'd also like to do something like this:
+
+	use Set::CrossProduct qw(setmap);
+
+	# use setmap with an existing Set::CrossProduct object
+	my @array = setmap { ... code ... } $iterator;
+
+	# use setmap with unnamed arrays
+	my @array = setmap { [ $_[0], $_[1] ] }
+		key => ARRAYREF, key2 => ARRAYREF;
+
+	# use setmap with named arrays
+	my @array = setmap { [ $key1, $key2 ] }
+		key => ARRAYREF, key2 => ARRAYREF;
+
+	# call apply() with a coderef. If the object had labels
+	# (constructed with a hash), you can use those labels in
+	# the coderef.
+	$set->apply( CODEREF );
+
 =head1 BUGS
 
 * none that i know about (yet)
@@ -492,9 +541,11 @@ members of the project can shepherd this module appropriately.
 
 brian d foy, C<< <bdfoy@cpan.org> >>
 
+Matt Miller implemented the named sets feature.
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2001-2014 brian d foy.  All rights reserved.
+Copyright © 2001-2016, brian d foy <bdfoy@cpan.org>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
